/*  Note: You are free to use whatever license you want.
    Eventually you will be able to edit it within Glade. */

/*  gulog
 *  Copyright (C) <YEAR> <AUTHORS>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include "gulogsig.h"
#include "gulog.h"

   GtkWidget*
   get_widget                             (GtkWidget       *widget,
   gchar           *widget_name)
   {
      GtkWidget *parent, *found_widget;
   
      for (;;)
      {
         if (GTK_IS_MENU (widget))
            parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
         else
            parent = widget->parent;
         if (parent == NULL)
            break;
         widget = parent;
      }
   
      found_widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (widget),
         widget_name);
      if (!found_widget)
         g_warning ("Widget not found: %s", widget_name);
      return found_widget;
   }

/* This is an internally used function to set notebook tab widgets. */
   void
   set_notebook_tab                       (GtkWidget       *notebook,
   gint             page_num,
   GtkWidget       *widget)
   {
      GtkNotebookPage *page;
      GtkWidget *notebook_page;
   
      page = (GtkNotebookPage*) g_list_nth (GTK_NOTEBOOK (notebook)->children, page_num)->data;
      notebook_page = page->child;
      gtk_widget_ref (notebook_page);
      gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page_num);
      gtk_notebook_insert_page (GTK_NOTEBOOK (notebook), notebook_page,
         widget, page_num);
      gtk_widget_unref (notebook_page);
   }

   static GList *pixmaps_directories = NULL;

/* Use this function to set the directory containing installed pixmaps. */
   void
   add_pixmap_directory                   (gchar           *directory)
   {
      pixmaps_directories = g_list_prepend (pixmaps_directories, g_strdup (directory));
   }

/* This is an internally used function to check if a pixmap file exists. */
#ifndef G_DIR_SEPARATOR_S
#define G_DIR_SEPARATOR_S "/"
#endif
   gchar*
   check_file_exists                      (gchar           *directory,
   gchar           *filename)
   {
      gchar *full_filename;
      struct stat s;
      gint status;
   
      full_filename = g_malloc (strlen (directory) + 1 + strlen (filename) + 1);
      strcpy (full_filename, directory);
      strcat (full_filename, G_DIR_SEPARATOR_S);
      strcat (full_filename, filename);
   
      status = stat (full_filename, &s);
      if (status == 0 && S_ISREG (s.st_mode))
         return full_filename;
      g_free (full_filename);
      return NULL;
   }

/* This is an internally used function to create pixmaps. */
   GtkWidget*
   create_pixmap                          (GtkWidget       *widget,
   gchar           *filename)
   {
      gchar *found_filename = NULL;
      GdkColormap *colormap;
      GdkPixmap *gdkpixmap;
      GdkBitmap *mask;
      GtkWidget *pixmap;
      GList *elem;
   
   /* We first try any pixmaps directories set by the application. */
      elem = pixmaps_directories;
      while (elem)
      {
         found_filename = check_file_exists ((gchar*)elem->data, filename);
         if (found_filename)
            break;
         elem = elem->next;
      }
   
   /* If we haven't found the pixmap, try the source directory. */
      if (!found_filename)
      {
         found_filename = check_file_exists ("pixmaps", filename);
      }
   
      if (!found_filename)
      {
         g_print ("Couldn't find pixmap file: %s", filename);
         return NULL;
      }
   
      colormap = gtk_widget_get_colormap (widget);
      gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask,
         NULL, found_filename);
      g_free (found_filename);
      if (gdkpixmap == NULL)
         return NULL;
      pixmap = gtk_pixmap_new (gdkpixmap, mask);
      gdk_pixmap_unref (gdkpixmap);
      gdk_bitmap_unref (mask);
      return pixmap;
   }

   GtkWidget*
   create_main_win ()
   {
      GtkWidget *main_win;
      GtkWidget *vbox;
      GtkWidget *frame1;
      GtkWidget *fixed1;
      GtkWidget *label1;
      GtkWidget *label8;
      GtkWidget *label9;
      GtkWidget *label7;
      GtkWidget *labelsbytes;
   
      GtkWidget *frame2;
      GtkWidget *fixed2;
      GtkWidget *label14;
      GtkWidget *label16;
      GtkWidget *label17;
      GtkWidget *label13;
      GtkWidget *labelrfile;
      GtkWidget *labelrbytes;
   
      GtkWidget *hbox1;
   
      GtkAccelGroup *accel_group;
      GtkTooltips *tooltips;
   
      tooltips = gtk_tooltips_new ();
   
      main_win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
      gtk_object_set_data (GTK_OBJECT (main_win), "main_win", main_win);
      gtk_widget_set_usize (main_win, 500, 400);
      gtk_window_set_title (GTK_WINDOW (main_win), "Gulog");
      gtk_window_set_policy (GTK_WINDOW (main_win), FALSE, FALSE, FALSE);
   
      vbox = gtk_vbox_new (FALSE, 0);
      gtk_object_set_data (GTK_OBJECT (main_win), "vbox", vbox);
      gtk_widget_show (vbox);
      gtk_container_add (GTK_CONTAINER (main_win), vbox);
   
      frame1 = gtk_frame_new ("Sending");
      gtk_object_set_data (GTK_OBJECT (main_win), "frame1", frame1);
      gtk_widget_show (frame1);
      gtk_box_pack_start (GTK_BOX (vbox), frame1, TRUE, TRUE, 0);
      /*gtk_container_set_border_width (GTK_CONTAINER (frame1), 2);*/
   
      fixed1 = gtk_fixed_new ();
      gtk_object_set_data (GTK_OBJECT (main_win), "fixed1", fixed1);
      gtk_widget_show (fixed1);
      gtk_container_add (GTK_CONTAINER (frame1), fixed1);
   
      sCurFile = gtk_text_new (NULL, NULL);
      gtk_object_set_data (GTK_OBJECT (main_win), "sCurFile", sCurFile);
      gtk_widget_show (sCurFile);
      gtk_fixed_put (GTK_FIXED (fixed1), sCurFile, 72, 12);
      gtk_widget_set_usize (sCurFile, 120, 20);
      GTK_WIDGET_UNSET_FLAGS (sCurFile, GTK_CAN_FOCUS);
      gtk_tooltips_set_tip (tooltips, sCurFile, "Curent sending file", NULL);
   
      label1 = gtk_label_new ("Curent file");
      gtk_object_set_data (GTK_OBJECT (main_win), "label1", label1);
      gtk_widget_show (label1);
      gtk_fixed_put (GTK_FIXED (fixed1), label1, 0, 16);
      gtk_widget_set_usize (label1, 64, 16);
      gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);
   
      labelsbytes = gtk_label_new ("Bytes sent");
      gtk_object_set_data (GTK_OBJECT (main_win), "labelsbytes", labelsbytes);
      gtk_widget_show(labelsbytes);
      gtk_fixed_put(GTK_FIXED(fixed1), labelsbytes, 235, 16);
   
      sBytes = gtk_text_new (NULL, NULL);
      gtk_object_set_data (GTK_OBJECT (main_win), "sBytes", sBytes);
      gtk_widget_show (sBytes);
      gtk_fixed_put (GTK_FIXED (fixed1), sBytes, 300, 12);
      gtk_widget_set_usize (sBytes, 180, 20);
      GTK_WIDGET_UNSET_FLAGS (sBytes, GTK_CAN_FOCUS);
      gtk_tooltips_set_tip (tooltips, sBytes, "Total bytes sent by now", NULL);	
   
      label7 = gtk_label_new ("Transfering");
      gtk_object_set_data (GTK_OBJECT (main_win), "label7", label7);
      gtk_widget_show (label7);
      gtk_fixed_put (GTK_FIXED (fixed1), label7, 0, 51);
      gtk_widget_set_usize (label7, 64, 16);
      gtk_label_set_justify (GTK_LABEL (label7), GTK_JUSTIFY_LEFT);
   
      sProgress = gtk_progress_bar_new ();
      gtk_object_set_data (GTK_OBJECT (main_win), "sProgress", sProgress);
      gtk_widget_show (sProgress);
      gtk_fixed_put (GTK_FIXED (fixed1), sProgress, 72, 45);
      gtk_widget_set_usize (sProgress, 408, 24);
      gtk_progress_set_show_text (GTK_PROGRESS (sProgress), TRUE);
      gtk_progress_configure(GTK_PROGRESS (sProgress), 0, 0, 0);
      gtk_progress_set_format_string(GTK_PROGRESS (sProgress), "Sent %p%% - %v/%u files");
   
      sCurRate = gtk_text_new (NULL, NULL);
      gtk_object_set_data (GTK_OBJECT (main_win), "sCurRate", sCurRate);
      gtk_widget_show (sCurRate);
      gtk_fixed_put (GTK_FIXED (fixed1), sCurRate, 72, 82);
      gtk_widget_set_usize (sCurRate, 120, 20);
      GTK_WIDGET_UNSET_FLAGS (sCurRate, GTK_CAN_FOCUS);
      gtk_tooltips_set_tip (tooltips, sCurRate, "The last sending transfer rate", NULL);
   
      sTotalRate = gtk_text_new (NULL, NULL);
      gtk_object_set_data (GTK_OBJECT (main_win), "sTotalRate", sTotalRate);
      gtk_widget_show (sTotalRate);
      gtk_fixed_put (GTK_FIXED (fixed1), sTotalRate, 300, 82);
      gtk_widget_set_usize (sTotalRate, 180, 20);
      GTK_WIDGET_UNSET_FLAGS (sTotalRate, GTK_CAN_FOCUS);
      gtk_tooltips_set_tip (tooltips, sTotalRate, "Total sending transfer rate", NULL);
   
      label8 = gtk_label_new ("Last bps");
      gtk_object_set_data (GTK_OBJECT (main_win), "label8", label8);
      gtk_widget_show (label8);
      gtk_fixed_put (GTK_FIXED (fixed1), label8, 10, 86);
   
      label9 = gtk_label_new ("Total bps");
      gtk_object_set_data (GTK_OBJECT (main_win), "label9", label9);
      gtk_widget_show (label9);
      gtk_fixed_put (GTK_FIXED (fixed1), label9, 240, 86);
   
      frame2 = gtk_frame_new ("Receving");
      gtk_object_set_data (GTK_OBJECT (main_win), "frame2", frame2);
      gtk_widget_show (frame2);
      gtk_box_pack_start (GTK_BOX (vbox), frame2, TRUE, TRUE, 0);
      /*gtk_container_set_border_width (GTK_CONTAINER (frame2), 2);*/
   
      fixed2 = gtk_fixed_new ();
      gtk_object_set_data (GTK_OBJECT (main_win), "fixed2", fixed2);
      gtk_widget_show (fixed2);
      gtk_container_add (GTK_CONTAINER (frame2), fixed2);
   
      label14 = gtk_label_new ("Transfering");
      gtk_object_set_data (GTK_OBJECT (main_win), "label14", label14);
      gtk_widget_show (label14);
      gtk_fixed_put (GTK_FIXED (fixed2), label14, 0, 51);
      gtk_widget_set_usize (label14, 64, 16);
      gtk_label_set_justify (GTK_LABEL (label14), GTK_JUSTIFY_LEFT);
   
      rProgress = gtk_progress_bar_new ();
      gtk_object_set_data (GTK_OBJECT (main_win), "rProgress", rProgress);
      gtk_widget_show (rProgress);
      gtk_fixed_put (GTK_FIXED (fixed2), rProgress, 72, 45);
      gtk_widget_set_usize (rProgress, 408, 24);
      gtk_progress_set_show_text (GTK_PROGRESS (rProgress), TRUE);
      gtk_progress_configure(GTK_PROGRESS (rProgress), 0, 0, 0);
      gtk_progress_set_format_string(GTK_PROGRESS (rProgress),
         "%p%% - %v bytes of %u");
   
      labelrfile = gtk_label_new ("Status");
      gtk_object_set_data (GTK_OBJECT (main_win), "labelrfile", label14);
      gtk_widget_show (labelrfile);
      gtk_fixed_put (GTK_FIXED (fixed2), labelrfile, 0, 76);
      gtk_widget_set_usize (labelrfile, 64, 16);
      gtk_label_set_justify (GTK_LABEL (labelrfile), GTK_JUSTIFY_LEFT);
   
      rFileProgress = gtk_progress_bar_new ();
      gtk_object_set_data (GTK_OBJECT (main_win), "rFileProgress", rFileProgress);
      gtk_widget_show (rFileProgress);
      gtk_fixed_put (GTK_FIXED (fixed2), rFileProgress, 72, 72);
      gtk_widget_set_usize (rFileProgress, 408, 24);
      gtk_tooltips_set_tip (tooltips, rFileProgress, "Status", NULL);
      gtk_progress_set_show_text (GTK_PROGRESS (rFileProgress), TRUE);
      gtk_progress_configure(GTK_PROGRESS (rFileProgress), 0, 0, 0);
      gtk_progress_set_format_string(GTK_PROGRESS (rFileProgress),
         "Received %p%% - %v/%u files");
   
      label13 = gtk_label_new ("Curent file");
      gtk_object_set_data (GTK_OBJECT (main_win), "label13", label13);
      gtk_widget_show (label13);
      gtk_fixed_put (GTK_FIXED (fixed2), label13, 0, 16);
      gtk_widget_set_usize (label13, 64, 16);
      gtk_label_set_justify (GTK_LABEL (label13), GTK_JUSTIFY_LEFT);
   
      rCurFile = gtk_text_new (NULL, NULL);
      gtk_object_set_data (GTK_OBJECT (main_win), "rCurFile", rCurFile);
      gtk_widget_show (rCurFile);
      gtk_fixed_put (GTK_FIXED (fixed2), rCurFile, 72, 12);
      gtk_widget_set_usize (rCurFile, 120, 20);
      GTK_WIDGET_UNSET_FLAGS (rCurFile, GTK_CAN_FOCUS);
      gtk_tooltips_set_tip (tooltips, rCurFile, "Curent receiving file", NULL);
   
      labelrbytes = gtk_label_new ("Bytes received");
      gtk_object_set_data (GTK_OBJECT (main_win), "labelrbytes", labelrbytes);
      gtk_widget_show(labelrbytes);
      gtk_fixed_put(GTK_FIXED(fixed2), labelrbytes, 210, 16);
   
      rBytes = gtk_text_new (NULL, NULL);
      gtk_object_set_data (GTK_OBJECT (main_win), "rBytes", rBytes);
      gtk_widget_show (rBytes);
      gtk_fixed_put (GTK_FIXED (fixed2), rBytes, 300, 12);
      gtk_widget_set_usize (rBytes, 180, 20);
      GTK_WIDGET_UNSET_FLAGS (rBytes, GTK_CAN_FOCUS);
      gtk_tooltips_set_tip (tooltips, rBytes, "Total bytes received by now", NULL);
   
      rCurRate = gtk_text_new (NULL, NULL);
      gtk_object_set_data (GTK_OBJECT (main_win), "rCurRate", rCurRate);
      gtk_widget_show (rCurRate);
      gtk_fixed_put (GTK_FIXED (fixed2), rCurRate, 72, 110);
      gtk_widget_set_usize (rCurRate, 120, 20);
      GTK_WIDGET_UNSET_FLAGS (rCurRate, GTK_CAN_FOCUS);
      gtk_tooltips_set_tip (tooltips, rCurRate, "The last receiving transfer rate", NULL);
   
      rTotalRate = gtk_text_new (NULL, NULL);
      gtk_object_set_data (GTK_OBJECT (main_win), "rTotalRate", rTotalRate);
      gtk_widget_show (rTotalRate);
      gtk_fixed_put (GTK_FIXED (fixed2), rTotalRate, 300, 110);
      gtk_widget_set_usize (rTotalRate, 180, 20);
      GTK_WIDGET_UNSET_FLAGS (rTotalRate, GTK_CAN_FOCUS);
      gtk_tooltips_set_tip (tooltips, rTotalRate, "Total receiving transfer rate", NULL);
   
      label16 = gtk_label_new ("Last bps");
      gtk_object_set_data (GTK_OBJECT (main_win), "label16", label16);
      gtk_widget_show (label16);
      gtk_fixed_put (GTK_FIXED (fixed2), label16, 10, 114);
   
      label17 = gtk_label_new ("Total bps");
      gtk_object_set_data (GTK_OBJECT (main_win), "label17", label17);
      gtk_widget_show (label17);
      gtk_fixed_put (GTK_FIXED (fixed2), label17, 240, 114);
   
      hbox1 = gtk_hbox_new (FALSE, 0);
      gtk_object_set_data (GTK_OBJECT (vbox), "hbox1", hbox1);
      gtk_widget_show (hbox1);
      gtk_box_pack_start (GTK_BOX (vbox), hbox1, TRUE, TRUE, 0);
      /*gtk_container_set_border_width (GTK_CONTAINER (hbox1), 2);*/
   
      helpBut = gtk_button_new_with_label ("Help");
      gtk_object_set_data (GTK_OBJECT (main_win), "helpBut", helpBut);
      gtk_widget_show (helpBut);
      gtk_box_pack_start (GTK_BOX (hbox1), helpBut, TRUE, TRUE, 0);
      /*gtk_fixed_put (GTK_FIXED (fixedp), helpBut, 149, 15);
      gtk_widget_set_usize (helpBut, 100, 40);*/
      gtk_tooltips_set_tip (tooltips, helpBut, "Click here (or press Ctrl+H) to get some help about Gulog.", NULL);
      gtk_signal_connect (GTK_OBJECT (helpBut), "clicked",
         GTK_SIGNAL_FUNC (on_helpBut_clicked),
         NULL);
      accel_group = gtk_accel_group_new ();
      gtk_window_add_accel_group (GTK_WINDOW (main_win), accel_group);
      gtk_widget_add_accelerator (helpBut, "clicked", accel_group,
         GDK_h, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
   
      exitBut = gtk_button_new_with_label ("Exit");
      gtk_object_set_data (GTK_OBJECT (main_win), "exitBut", exitBut);
      gtk_widget_show (exitBut);
      gtk_box_pack_start (GTK_BOX (hbox1), exitBut, TRUE, TRUE, 0);
      /*gtk_fixed_put (GTK_FIXED (fixedp), exitBut, 251, 15);
      gtk_widget_set_usize (exitBut, 100, 40);*/
      gtk_tooltips_set_tip (tooltips, exitBut, "Click here (or press Ctrl+Q) to exit, isn't obviously?", NULL);
      gtk_signal_connect (GTK_OBJECT (exitBut), "clicked",
         GTK_SIGNAL_FUNC (on_exitBut_clicked),
         NULL);
      gtk_widget_add_accelerator (exitBut, "clicked", accel_group,
         GDK_q, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
   
      statusbar = gtk_statusbar_new ();
      gtk_object_set_data (GTK_OBJECT (main_win), "statusbar", statusbar);
      gtk_widget_show (statusbar);
      sb_id=gtk_statusbar_get_context_id( GTK_STATUSBAR(statusbar), "Statusbar");
      gtk_box_pack_start (GTK_BOX (vbox), statusbar, FALSE, TRUE, 0);
   
      gtk_object_set_data (GTK_OBJECT (main_win), "tooltips", tooltips);
   
      return main_win;
   }

