/*  gulog - GTK uucp log & traffic analyzer
 *  Copyright (C) 2000, Mircea Mitu
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
/* #include <gtk/gtkintl.h> */
#define __USE_BSD

#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <fcntl.h>
#include <dirent.h>
#include <time.h>
#include <string.h>
#include "gulog.h"
#include "gulogsig.h"


   typedef struct{
      FILE *f;
      char *name;
      time_t t;
   } filechg;

   typedef struct{
      gint ready, to, total;
   } statistic;

   filechg Log, Stats;
   long rSizeOfFile=0;
   statistic sStat, rStat;
   gchar *spooldir;

	#define other 0
	#define receiving 1
	#define sending 2

   gint curAction=other;
   gint timeout_sending;
   time_t callingTime=0, curTime=0;
   int rcount=0, scount=0, rBPS=0, sBPS=0;
   long int rbytes=0, sbytes=0;

   GtkWidget *HelpDlg=NULL;


   int selectll(const struct dirent *dd)
   {
      if ( strcmp(dd->d_name,".")!=0 && strcmp(dd->d_name,"..")!=0)
         return 1;
      return 0;
   }

   int selectsend(const struct dirent *dd)
   {
      if ( strcmp(dd->d_name,".")!=0 && strcmp(dd->d_name,"..")!=0
      && strlen(dd->d_name)==6)
         return 1;
      return 0;
   }

   int selectsys(const struct dirent *dd)
   {
      return(dd->d_name[0]!='.') ;
   }

   int sortll(const struct dirent **a,const struct dirent **b)
   {
      struct stat bufa, bufb;
   
      stat(a[0]->d_name,&bufa);
      stat(b[0]->d_name,&bufb);
      if (bufa.st_ctime<bufb.st_ctime) 
         return -1;
      else if (bufa.st_ctime>bufb.st_ctime) 
         return 1;
      return 0;
   }

   void setfilechg(filechg *fc, char *s)
   {
      if(fc->f!=NULL)fclose(fc->f);
      fc->f=fopen(s,"r");
      if(fc->f==NULL)
         g_error("Can't open %s\n",s);
      fc->name=(char *)g_malloc(strlen(s)+1);
      strcpy(fc->name,s);
      fc->t=-1;
   }

   int fileisnew(filechg *fc)
   {
      struct stat buf;
   
      stat(fc->name,&buf);
      if(buf.st_mtime!=fc->t){
         fc->t=buf.st_mtime;
         return 1;
      }
      return 0;
   }

   void
   readfolder ()
   {
      struct dirent **namelist;
      struct stat buf;
      int n;
      char s[20];
   
      chdir(tempdir);
      n = scandir (".", &namelist, selectll, sortll);
      if (n < 0)
         g_error ("scandir failed\n");
      else {
         gtk_text_backward_delete( GTK_TEXT(rCurFile), 
            gtk_text_get_length(GTK_TEXT(rCurFile)));
      
         if (n){
            n--;
            strcpy (s, namelist[n]->d_name);
            stat (s, &buf);
            rStat.to=n;
            gtk_text_insert( GTK_TEXT(rCurFile), NULL, NULL, NULL,
               s,-1);
            gtk_progress_configure(GTK_PROGRESS(rProgress),
               buf.st_size,0,rSizeOfFile);
            gtk_progress_configure(GTK_PROGRESS(rFileProgress),
               rStat.ready,0,rStat.ready+rStat.to);
            sprintf(s, "%ld", rbytes+buf.st_size);
            gtk_text_backward_delete( GTK_TEXT(rBytes), 
               gtk_text_get_length(GTK_TEXT(rBytes)));
            gtk_text_insert( GTK_TEXT(rBytes), NULL, NULL, NULL, s, -1);
         }
      
      }
      chdir(sysdir);
      n = scandir (".", &namelist, selectsend, 0);
      if (n < 0)
         g_error ("sysdir: %s - scandir failed\n", sysdir);
      else {
         sStat.to=n;
         gtk_progress_configure( GTK_PROGRESS (sProgress),
            sStat.ready, 0, sStat.to+sStat.ready);
      }
   }

   gint timeout_sprogress(gpointer data)
   {
      gfloat val=gtk_progress_get_value( GTK_PROGRESS (sProgress));
   
      if (val>=100)
         val=0;
      gtk_progress_set_value( GTK_PROGRESS (sProgress),val+1);
      return(TRUE);
   }

   void updateAction(gint action)
   {
      if (curAction==action)
         return;
      switch(curAction){
         case receiving:
            break;
         case sending:
            gtk_timeout_remove(timeout_sending);
            gtk_progress_set_activity_mode( GTK_PROGRESS (sProgress), FALSE);
            gtk_progress_configure( GTK_PROGRESS (sProgress),
               sStat.ready, 0, sStat.to+sStat.ready);
            break;
         case other:
            break;
      }
      curAction=action;
      switch(curAction){
         case receiving:
         
            break;
         case sending:
            timeout_sending=gtk_timeout_add(100, timeout_sprogress, NULL);
            break;
         case other:
            break;
      }
   }


   void parseStatsLine(char *str)
   {
      char s1[20], s2[20], s3[20], *str1;
      struct tm curtm;
      int sbps, rbps;
      long int bytes;
   
      sscanf(str, "%s %s (%d-%d-%d %d:%d:%d",
         s1, s1, 
         &curtm.tm_year, &curtm.tm_mon, &curtm.tm_mday, 
         &curtm.tm_hour, &curtm.tm_min, &curtm.tm_sec);
      curtm.tm_mon--;
      curtm.tm_year-=1970;
      curtm.tm_isdst=-1;
      curTime=mktime(&curtm);
      if(difftime(curTime, callingTime)>0){
         str1=strchr(str,')'); 
         str1++;
         sscanf(str1, "%s", s1);
         if(strcmp(s1, "received")==0){
            rcount++;
            sscanf(str1, "%s %s %s %s %s %s (%d",
               s1, s1, s1, s1, s1, s1, &rbps);
            rBPS+=rbps;
            sscanf(str1, "%s %ld", s1, &bytes);
            rbytes+=bytes;
            sprintf(s1, " %ld", rbytes);
            sprintf(s2, " %d", rbps);
            sprintf(s3, " %d", rBPS/rcount);
            gtk_text_backward_delete( GTK_TEXT(rBytes), 
               gtk_text_get_length(GTK_TEXT(rBytes)));
            gtk_text_insert( GTK_TEXT(rBytes), NULL, NULL, NULL, s1, -1);
            gtk_text_backward_delete( GTK_TEXT(rCurRate), 
               gtk_text_get_length(GTK_TEXT(rCurRate)));
            gtk_text_insert( GTK_TEXT(rCurRate), NULL, NULL, NULL, s2, -1);
            gtk_text_backward_delete( GTK_TEXT(rTotalRate), 
               gtk_text_get_length(GTK_TEXT(rTotalRate)));
            gtk_text_insert( GTK_TEXT(rTotalRate), NULL, NULL, NULL, s3, -1);
         } 
         else if (strcmp(s1, "sent")==0){
            scount++;
            sscanf(str1, "%s %s %s %s %s %s (%d",
               s1, s1, s1, s1, s1, s1, &sbps);
            sBPS+=sbps;
            sscanf(str1, "%s %ld", s1, &bytes);
            sbytes+=bytes;
            sprintf(s1, " %ld", sbytes);
            sprintf(s2, " %d", sbps);
            sprintf(s3, " %d", sBPS/scount);
            gtk_text_backward_delete( GTK_TEXT(sBytes), 
               gtk_text_get_length(GTK_TEXT(sBytes)));
            gtk_text_insert( GTK_TEXT(sBytes), NULL, NULL, NULL, s1, -1);
            gtk_text_backward_delete( GTK_TEXT(sCurRate), 
               gtk_text_get_length(GTK_TEXT(sCurRate)));
            gtk_text_insert( GTK_TEXT(sCurRate), NULL, NULL, NULL, s2, -1);
            gtk_text_backward_delete( GTK_TEXT(sTotalRate), 
               gtk_text_get_length(GTK_TEXT(sTotalRate)));
            gtk_text_insert( GTK_TEXT(sTotalRate), NULL, NULL, NULL, s3, -1);
         }
      }
   }

   void parseLogLine(char *str)
   {
      char s1[20], s2[30], s3[20], s[200], ss[200], *str2, *str3;
      struct tm curtm;
   
      str2=strchr(str,')');
      str2++;
      sscanf(str2,"%s", s);
      strcpy(ss,s);
      if(strcmp(ss,"Receiving")==0){
         updateAction(receiving);
         rStat.ready++;
         gtk_progress_configure(GTK_PROGRESS(rFileProgress),
            rStat.ready,0,rStat.ready+rStat.to);
         str3=strrchr(str,')');
         str3--;
         if(isdigit(str3[0]))
            sscanf(str2,"%s %s %s %s %s %s %s %s",
               s1, s1, s, s2, s1, s1, s1, s3);
         else 
            sscanf(str2, "%s %s %s %s %s", s1, s1, s, s2, s1);
         s2[0]=' ';
         rSizeOfFile=atol(s2);
      }
      else if(strcmp(ss,"Sending")==0){
         updateAction(sending);
         sStat.ready++;
         str3=strstr(str2, "(D.");
         sscanf(str3, "%s", s3);
         s3[0]=' ';
         s3[7]=' ';
         gtk_text_backward_delete( GTK_TEXT(sCurFile), 
            gtk_text_get_length(GTK_TEXT(sCurFile)));
         gtk_text_insert( GTK_TEXT(sCurFile), NULL, NULL, NULL, s3, -1);
         gtk_progress_configure(GTK_PROGRESS(sProgress),1,0,100);
         gtk_progress_set_value( GTK_PROGRESS (sProgress), 0);
         gtk_progress_set_activity_mode( GTK_PROGRESS (sProgress), TRUE);
         gtk_progress_bar_set_activity_step( GTK_PROGRESS_BAR (sProgress), 20);
      }
      else if(strcmp(ss,"Calling")==0){
         updateAction(other);
         sStat.ready=rStat.ready=0;
         gtk_progress_configure(GTK_PROGRESS(rFileProgress),
            rStat.ready,0,rStat.ready+rStat.to);
         sscanf(str, "%s %s %s (%d-%d-%d %d:%d:%d",
            s1, s1, s1, 
            &curtm.tm_year, &curtm.tm_mon, &curtm.tm_mday, 
            &curtm.tm_hour, &curtm.tm_min, &curtm.tm_sec);
         curtm.tm_mon--;
         curtm.tm_year-=1970;
         curtm.tm_isdst=-1;
         callingTime=mktime(&curtm);
         rcount=scount=0;
         rbytes=sbytes=0;
      }
      else{
         updateAction(other);
      }
      strcpy(ss,str2);
      ss[strlen(ss)-1]=' ';
      gtk_statusbar_push( GTK_STATUSBAR(statusbar), sb_id, ss);
   }

   gint timeout_treat(gpointer data)
   {
      char s[200];
      long pos;
   
      if(fileisnew(&Log)){
         pos=ftell(Log.f);
         fclose(Log.f);
         Log.f=fopen(Log.name,"r");
         fseek(Log.f, pos, SEEK_SET);
         while(!feof(Log.f))
            if(NULL!=fgets(s,190,Log.f))
               parseLogLine(s);
      }
      readfolder();
      if(fileisnew(&Stats)){
         pos=ftell(Stats.f);
         fclose(Stats.f);
         Stats.f=fopen(Stats.name,"r");
         fseek(Stats.f, pos, SEEK_SET);
         while(!feof(Stats.f))
            if(NULL!=fgets(s,190,Stats.f))
               parseStatsLine(s);
      }
      return (TRUE);
   }

   void getLogFile()
   {
      FILE *f;
   
      f=fopen("/var/log/uucp/Log", "r");
      if(f==NULL){
         f=fopen("/var/spool/uucp/Log", "r");
         if(f==NULL){
            f=fopen("/usr/spool/uucp/Log", "r");
            if(f==NULL)
               g_error("Could not open Log file. Not in:\n%s\n%s\n%s\n",
                  "/var/log/uucp/Log", "/var/spool/uucp/Log", "/usr/spool/uucp/Log");
         } 
         else
            logfile=strdup("/var/spool/uucp/Log");
      } 
      else
         logfile=strdup("/var/log/uucp/Log");
      printf("Log: %s\n", logfile);
   }

   void getStatFile()
   {
      FILE *f;
   
      f=fopen("/var/log/uucp/Stats", "r");
      if(f==NULL){
         f=fopen("/var/spool/uucp/Stats", "r");
         if(f==NULL){
            f=fopen("/usr/spool/uucp/Stats", "r");
            if(f==NULL)
               g_error("Could not open Stats file. Not in:\n%s\n%s\n%s\n",
                  "/var/log/uucp/Stats", "/var/spool/uucp/Stats", "/usr/spool/uucp/Stats");
         } 
         else
            statfile=strdup("/var/spool/uucp/Stats");
      } 
      else
         statfile=strdup("/var/log/uucp/Stats");
      printf("Stats: %s\n", statfile);
   }

   void getSpoolDir()
   {
      if(chdir("/var/spool/uucp")==-1){
         if(chdir("/usr/spool/uucp")==-1)
            g_error("Could not open uucp spool dir. Not in:\n%s\n%s\n",
               "/var/spool/uucp",  "/usr/spool/uucp");
         else spooldir=strdup("/usr/spool/uucp");
      }
      else spooldir=strdup("/var/spool/uucp");
   
      printf("Spool dir: %s\n", spooldir);
   }

   void getTempDir()
   {
      struct dirent **namelist;
      int n;
      char s[120];
   
   
      chdir(spooldir);
      if(chdir(".Temp")==-1)
         g_error("Could not change dir to %s/.Temp\n", spooldir);
      else{
         n = scandir (".", &namelist, selectsys, alphasort);
         if (n < 0)
            g_error ("sys dir scandir failed\n");
         else {
            strcpy(s, spooldir);
            strcat(s, "/.Temp/");
            strcat(s, namelist[0]->d_name);
            tempdir=strdup(s);
         }
      
         printf("Temp dir: %s\n", tempdir);
      }
   }	

   void getSysDir()
   {
   
      struct dirent **namelist;
      int n;
      char s[120];
   
   
      chdir(spooldir);
      n = scandir (".", &namelist, selectsys, alphasort);
      if (n < 0)
         g_error ("sys dir scandir failed\n");
      else {
         strcpy(s, spooldir);
         strcat(s, "/");
         strcat(s, namelist[0]->d_name);
         strcat(s, "/D.");
         sysdir=strdup(s);
         if(chdir(sysdir)==-1)
            g_error("Change dir to sysdir=%s failed\n", sysdir);
      }
   
      printf("Sys dir: %s\n", sysdir);
   }

   int
   main (int argc, char *argv[])
   {
      GtkWidget *main_win;
      gint timeout;
   
      /*gtk_set_locale ();*/
      gtk_init (&argc, &argv);
      logfile=statfile=spooldir=tempdir=sysdir=NULL;
      getLogFile();
      getStatFile();
      getSpoolDir();
      getTempDir();
      getSysDir();
      Log.f=Stats.f=NULL;
      setfilechg(&Log, logfile);
      setfilechg(&Stats, statfile);
      sStat.ready=sStat.to=sStat.total=rStat.ready=rStat.to=rStat.total=0;
      main_win = create_main_win ();
      gtk_widget_show (main_win);
      gtk_statusbar_push( GTK_STATUSBAR(statusbar), sb_id, " Parsing log file... please wait");
      timeout=gtk_timeout_add(1000, timeout_treat, NULL);
      gtk_main ();
      gtk_timeout_remove(timeout);
      if(Stats.f!=NULL)
         fclose(Stats.f);
      if(Log.f!=NULL)
         fclose(Log.f);
      return 0;
   }

   void closeHelp(GtkObject *obj)
   {
      gtk_widget_destroy(GTK_WIDGET(obj));
      /*obj=NULL;*/
      HelpDlg=NULL;
   }


   void
   on_helpBut_clicked                     (GtkButton       *button,
   gpointer         user_data)
   {
      GtkWidget *but, *label, *infos, *sep;
      gchar *s, ss[400];
   
      if(HelpDlg!=NULL)
         return;
   
      HelpDlg=gtk_dialog_new();
      gtk_container_set_border_width (GTK_CONTAINER (HelpDlg), 10);
   
      but=gtk_button_new_with_label("Ok");
      gtk_signal_connect_object (GTK_OBJECT (but), "clicked",
         GTK_SIGNAL_FUNC (closeHelp),
         GTK_OBJECT (HelpDlg));
      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (HelpDlg)->action_area), but, TRUE, TRUE, 0);
      gtk_widget_show (but);
   
      label=gtk_label_new("This is Gulog 1.0\n\na GTK uucp log & traffic analyzer\n\
	written by Mircea Mitu\n\nCopyright (C) 2000\n");
      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (HelpDlg)->vbox), label, TRUE, TRUE, 0);
      gtk_widget_show (label);
   
      sep=gtk_hseparator_new();
      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (HelpDlg)->vbox), sep, TRUE, TRUE, 0);
      gtk_widget_show (sep);
   
      sprintf(ss, "\nLog: %s\nStats: %s\nSys dir: %s\nTemp dir: %s\n", logfile, statfile, sysdir, tempdir);
      s=strdup(ss);
      infos=gtk_label_new(s);
   	gtk_label_set_justify(GTK_LABEL(infos), GTK_JUSTIFY_LEFT);
      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (HelpDlg)->vbox), infos, TRUE, TRUE, 0);
      gtk_widget_show (infos);
   
      gtk_widget_show(HelpDlg);
   }


   void
   on_exitBut_clicked                     (GtkButton       *button,
   gpointer         user_data)
   {
      gtk_main_quit ();
   }

